﻿
local tParty = true
local tCastbar = true
local tRunebar = true

local fontn = "Interface\\AddOns\\oUF_alekk\\fonts\\CalibriBold.ttf"
local fontpixel = "Interface\\AddOns\\oUF_alekk\\fonts\\Calibri.ttf"
local texturebar = "Interface\\AddOns\\oUF_alekk\\textures\\Ruben"
local trunebar = "Interface\\AddOns\\oUF_alekk\\textures\\rothTex"
local textureborder = "Interface\\AddOns\\oUF_alekk\\textures\\Caith.tga"
local bubbleTex = 'Interface\\Addons\\oUF_alekk\\textures\\bubbleTex'
local cbborder = 'Interface\\AddOns\\oUF_alekk\\textures\\border'
local glowTexture = [=[Interface\Addons\aNamePlates\media\glowTex]=]
local mscale = 1

local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\AddOns\\oUF_alekk\\textures\\border", edgeSize = 12,
	insets = {left = 0, right = 0, top = 0, bottom = 0},
}

local backdrophp = {
	bgFile = "Interface\\AddOns\\oUF_alekk\\textures\\Ruben",
	insets = {left = 0, right = 0, top = 0, bottom = 0},
}

local colors = {
	green = { r = 0, g = 1, b = 0 },
	gray = { r = 0.5, g = 0.5, b = 0.5 },
	white = { r = 1, g = 1, b = 1},
	unknown = { r = .41, g = .95, b = .2 },
}
local classification = {
	worldboss = '%s |cffffd700Boss|r',
	rareelite = '%s |cffffd700R+|r',
	elite = '%s |cffffd700++|r',
	rare = '%s Rare',
	normal = '%s',
	trivial = '%s',
}

oUF.colors.power['MANA'] = {.30,.45,.65}
oUF.colors.power['RAGE'] = {.70,.30,.30}
oUF.colors.power['FOCUS'] = {.70,.45,.25}
oUF.colors.power['ENERGY'] = {.65,.65,.35}
oUF.colors.power['RUNIC_POWER'] = {.45,.45,.75}

oUF.colors.happiness = {
	[1] = {.69,.31,.31},
	[2] = {.65,.65,.30},
	[3] = {.33,.59,.33},
}

colors.runes = setmetatable({
		[1] = {0.69, 0.31, 0.31},
		[2] = {0.33, 0.59, 0.33},
		[3] = {0.31, 0.45, 0.63},
		[4] = {0.84, 0.75, 0.05},
},{__index = oUF.colors.runes})

oUF.colors.tapped = {.55,.57,.61}
oUF.colors.disconnected = {.5,.5,.5}

local setFontString = function(parent, fontStyle, fontHeight)
	local fs = parent:CreateFontString(nil, "OVERLAY")
	fs:SetFont(fontStyle, fontHeight)
	fs:SetShadowColor(0,0,0)
	fs:SetShadowOffset(1, -1)
	fs:SetTextColor(1,1,1)
	fs:SetJustifyH("LEFT")
	return fs
end

local kilo = function(value)
	if value >= 1e6 then
		return string.format("%.1fm", value/1e6)
	elseif value >= 1e3 then
		return string.format("%.1fk", value/1e3)
	else
		return value
	end	
end
--local CancelAura = function(self, button)
--	if button == "RightButton" and not self.debuff then
--		CancelUnitBuff("player", self:GetID())
--	end
--end
local getDifficultyColor = function(level)
	if level == '??' or level == '-1' then
		return  1,0.84,1 -- .69,.31,.31
	else
	local levelDiff = UnitLevel('target') - UnitLevel('player')
		if levelDiff >= 5 then
			return .69,.31,.31
		elseif levelDiff >= 3 then
			return .71,.43,.27
		elseif levelDiff >= -2 then
			return .84,.75,.65
		elseif -levelDiff <= GetQuestGreenRange() then
			return .33,.59,.33
		else
			return  .55,.57,.61
		end
	end
end

local function UpdateRuneBar(self, elapsed)
	local start, duration, ready = GetRuneCooldown(self:GetID())

	if(ready) then
		self:SetValue(1)
		self:SetScript('OnUpdate', nil)
	else
		self:SetValue((GetTime() - start) / duration)
	end
end

local function UpdateRunePower(self, event, rune, usable)
	for i = 1, 6 do
		if(rune == i and not usable and GetRuneType(rune)) then
			self.RuneBar[i]:SetScript('OnUpdate', UpdateRuneBar)
		end
	end
end

local function UpdateRuneType(self, event, rune)
	if(rune) then
		local runetype = GetRuneType(rune)
		if(runetype) then
			self.RuneBar[rune]:SetStatusBarColor(unpack(colors.runes[runetype]))
		end
	else
		for i = 1, 6 do
			local runetype = GetRuneType(i)
			if(runetype) then
				self.RuneBar[i]:SetStatusBarColor(unpack(colors.runes[runetype]))
				
			end
		end
	end
end

local function updateColor(self, unit, bar)
	local color, colorRGB
	local _, class = UnitClass(unit)

	if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) then
		color = {.5,.5,.5}
	elseif(UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit)) then
		color = {.5,.5,.5}
	elseif(unit =='pet' and GetPetHappiness()) then
		color = oUF.colors.happiness[GetPetHappiness()]
	elseif(UnitIsPlayer(unit)) then
		color = oUF.colors.class[class]
	else
		colorRGB = FACTION_BAR_COLORS[UnitReaction(unit, 'player')]	
	end

	if(color) then
		bar:SetStatusBarColor(color[1], color[2], color[3])	
		bar:SetBackdropColor(color[1]/2.5, color[2]/2.5, color[3]/2.5,0.8)
	elseif(colorRGB) then
		bar:SetStatusBarColor(colorRGB.r, colorRGB.g, colorRGB.b)	
		bar:SetBackdropColor(colorRGB.r/2.5, colorRGB.g/2.5, colorRGB.b/2.5, 0.8)
	end
end

local function OverrideUpdateName(self, event, unit)
	if(self.unit ~= unit or not self.Name) then return end
	if(unit == 'player') then
		self.Name:Hide()
	else
		self.Name:SetText(UnitName(unit))
	end

	if (UnitClassification(unit)== "worldboss" or UnitClassification(unit)== "rareelite" or UnitClassification(unit)== "elite") then
		self:SetBackdropBorderColor(1,0.84,0,1)
	else
		self:SetBackdropBorderColor(1,1,1,1)
	end	
	
	if(unit == 'player' or unit == 'target') then
		local level = not UnitIsConnected(unit) and '??' or UnitLevel(unit) < 1 and '??' or UnitLevel(unit)
		self.Lvl:SetFormattedText(classification[UnitClassification(unit)], level)


		if UnitCanAttack("player", unit) then
			self.Lvl:SetTextColor(getDifficultyColor(level))
		else
			self.Lvl:SetTextColor(1, 1, 1)
		end

		local color
		if UnitIsPlayer(unit) then	
			color = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
			if UnitClass(unit) == "Чернокнижница" or UnitClass(unit) == "Чернокнижник" then
				self.Class:SetText("Варлок")
			elseif UnitClass(unit) == "Рыцарь смерти" then
				self.Class:SetText("Р. смерти")
			else
				self.Class:SetText(UnitClass(unit))
			end
		else
			self.Class:SetText(UnitCreatureFamily(unit) or UnitCreatureType(unit))
		end
		if not color then color = colors.white end
		self.Class:SetVertexColor(color.r, color.g, color.b)
		self.Race:SetText(UnitRace(unit))
	end
end

local function PostUpdateHealth(self, event, unit, bar, min, max)
	if(not UnitIsConnected(unit)) then
		bar:SetValue(0)
		bar.value:SetText('|cffD7BEA5'..'Offline')
	elseif(UnitIsDead(unit)) then
		bar:SetValue(0)
		bar.value:SetText('|cffD7BEA5'..'Dead')
	elseif(UnitIsGhost(unit)) then
		bar:SetValue(0)
		bar.value:SetText('|cffD7BEA5'..'Ghost')
	else
		if(unit=="player") then
				bar.value:SetFormattedText("%d | %d | %d|cffffffff%%",min, max ,floor(min/max*100))
		else
				if unit=="target" then
					bar.value:SetFormattedText("%.1f|cffffffff%% | %s", (min/max*100),kilo(max))

				else
					bar.value:SetFormattedText("%d|cffffffff%%", floor(min/max*100))

				end
		end
	end
	updateColor(self.Name, unit, bar)
	self:UNIT_NAME_UPDATE(event, unit)
end

local function PostUpdatePower(self, event, unit, bar, min, max)
	local _, pType = UnitPowerType(unit)
	local color = self.colors.power[pType] or {0.7,0.7,0.7} 
	
	bar:SetStatusBarColor(color[1], color[2], color[3]) 
	bar:SetBackdropColor(color[1]/3, color[2]/3, color[3]/3,0.8)

    if max == 0 or UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) then  
        bar:SetValue(0) 
        if bar.value then
            bar.value:SetText()
        end
    elseif bar.value then
		if(unit=='player') then  
			bar.value:SetText(min .. ' | ' .. max)
		elseif(unit=="target") then
			bar.value:SetText(kilo(min) .. ' | ' .. kilo(max))
		else
			bar.value:SetText()  
		end
	end
	self:UNIT_NAME_UPDATE(event, unit)
end

local FormatTime = function(s)
	local DAY, HOUR, MINUTE = 86400, 3600, 60
	if s >= DAY then
		return format('%dd', floor(s/DAY + 0.5)), s % DAY
	elseif s >= HOUR then
		return format('%dh', floor(s/HOUR + 0.5)), s % HOUR
	elseif s >= MINUTE then
		if s <= MINUTE*3 then
			return format('%d:%02d', floor(s/60), s % MINUTE), s - floor(s)
		end
		return format('%dm', floor(s/MINUTE + 0.5)), s % MINUTE
	end
	return floor(s + 0.5), s - floor(s)
end

local CreateAuraTimer = function(self,elapsed)
	if self.timeLeft then
		self.elapsed = (self.elapsed or 0) + elapsed
		if self.elapsed >= 0.1 then
			if not self.first then
				self.timeLeft = self.timeLeft - self.elapsed
			else
				self.timeLeft = self.timeLeft - GetTime()
				self.first = false
			end
			if self.timeLeft > 60 then
				local time = FormatTime(self.timeLeft)
				self.remaining:SetText(time)
				self.remaining:SetTextColor(0.8, 0.8, 0.9)
			elseif self.timeLeft > 5 then
				local time = FormatTime(self.timeLeft)
				self.remaining:SetText(time)
				self.remaining:SetTextColor(0.8, 0.8, 0.2)
			elseif self.timeLeft > 0 then
				local time = FormatTime(self.timeLeft)
				self.remaining:SetText(time)
				self.remaining:SetTextColor(0.9, 0.3, 0.3)
			else
				self.remaining:Hide()
				self:SetScript("OnUpdate", nil)
			end
			self.elapsed = 0
		end
	end
end

local function PostCreateAuraIcon(self, button, icons, index, debuff)
	button.backdrop = CreateFrame("Frame", nil, button)
	button.backdrop:SetPoint("TOPLEFT", button, "TOPLEFT", -3.5, 3)
	button.backdrop:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 4, -3.5)
	button.backdrop:SetFrameStrata("BACKGROUND")
	button.backdrop:SetBackdrop {
		edgeFile = glowTex, edgeSize = 5,
		insets = {left = 3, right = 3, top = 3, bottom = 3}
	}
	button.backdrop:SetBackdropColor(0, 0, 0, 0)
	button.backdrop:SetBackdropBorderColor(0, 0, 0)

	button.count:SetPoint("BOTTOMRIGHT", 3,-3)
	button.count:SetJustifyH("RIGHT")
	if self.unit == "player" then
		button.count:SetFont(fontn, 17, "OUTLINE")
	else
		button.count:SetFont(fontn, 14, "OUTLINE")
	end
	button.count:SetTextColor(0.8, 0.8, 0.8)

	button.cd.noOCC = true
	button.cd.noCooldownCount = true
	icons.disableCooldown = true

	button.overlay:SetTexture(textureborder)
	button.overlay:SetPoint("TOPLEFT", button, "TOPLEFT", -1, 1)
	button.overlay:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 1, -1)
	button.overlay:SetTexCoord(0, 1, 0, 1)
	button.overlay.Hide = function(self) end

	button.remaining = setFontString(button, fontn, 12)
		button.remaining:SetFont(fontn, 14, "OUTLINE")
		if self.unit == "player" then
			button.remaining:SetFont(fontn, 17, "OUTLINE")
			button:SetScript("OnMouseUp", CancelAura)
		end
	if icons == self.Enchant then
		button.remaining:SetFont(fontn, 15, "OUTLINE")
		button.overlay:SetVertexColor(0.33, 0.59, 0.33)
	end
	button.remaining:SetPoint("CENTER", 1, -1)
end

local CreateEnchantTimer = function(self, icons)
	for i = 1, 2 do
		local icon = icons[i]
		if icon.expTime then
			icon.timeLeft = icon.expTime - GetTime()
			icon.remaining:Show()
		else
			icon.remaining:Hide()
		end
		icon:SetScript("OnUpdate", CreateAuraTimer)
	end
end

local function PostUpdateAuraIcon(self, icons, unit, icon, index, offset, filter, debuff)
	local _, _, _, _, _, duration, expirationTime, unitCaster, _ = UnitAura(unit, index, icon.filter)
	if (unitCaster == "player" or unitCaster == "pet" or unitCaster == "vehicle") and self.unit == "target" then
		if icon.debuff then
			icon.overlay:SetVertexColor(0.8, 0.2, 0.2)
		else
			icon.overlay:SetVertexColor(0.2, 0.8, 0.2)
		end
	else
		if UnitIsEnemy("player", unit) then
			if icon.debuff then
				icon.icon:SetDesaturated(true)
			end
		end
		icon.overlay:SetVertexColor(0.5, 0.5, 0.5)
	end

	if duration and duration > 0 then
		icon.remaining:Show()
	else
		icon.remaining:Hide()
	end

	icon.duration = duration
	icon.timeLeft = expirationTime
	icon.first = true
	icon:SetScript("OnUpdate", CreateAuraTimer)
end

local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub('(.)', string.upper, 1)

	if(unit == 'party') then
		ToggleDropDownMenu(1, nil, _G['PartyMemberFrame'..self.id..'DropDown'], 'cursor', 0, 0)
	elseif(_G[cunit..'FrameDropDown']) then
		ToggleDropDownMenu(1, nil, _G[cunit..'FrameDropDown'], 'cursor', 0, 0)
	end
end

local function OverrideCastbarTime(self, duration)
		if(self.channeling) then
			self.Time:SetFormattedText('%.1f / %.2f', self.max - duration, self.max)
		elseif(self.casting) then
			self.Time:SetFormattedText('%.1f / %.2f', duration, self.max)
		end	
end

local function OverrideCastbarDelay(self, duration)
		if(self.channeling) then
			self.Time:SetFormattedText('%.1f / %.2f |cffff0000+ %.1f', self.max - duration, self.max, self.delay)
		elseif(self.casting) then
			self.Time:SetFormattedText('%.1f / %.2f |cffff0000+ %.1f', duration, self.max, self.delay)
		end	
end



local function Style(self, unit)

	local _, class = UnitClass('player')
	self.disallowVehicleSwap = true
	self.menu = menu
	self:RegisterForClicks('AnyUp')
	self:SetAttribute('*type2', 'menu')
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	
	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0,0,0,1)
	if unit == "targettarget" or unit == "focus" or unit == "focustarget" then
		self:SetWidth(135)
		self:SetHeight(25)
	elseif unit == "player" or unit == "target" then
		self:SetWidth(275)
		self:SetHeight(47)
	else
		self:SetWidth(125)
		self:SetHeight(38)
	end
	
	self.Health = CreateFrame("StatusBar", nil, self)
	self.Health:SetStatusBarTexture(texturebar)
	self.Health:SetStatusBarColor(.31, .31, .31)
	self.Health:SetPoint("LEFT", 4.5,0)
	self.Health:SetPoint("RIGHT", -4.5,0)
	self.Health:SetPoint("TOP", 0,-4.5)
	self.Health:SetBackdrop(backdrophp)
	self.Health.Smooth = true
	self.Health.colorTapping = true
	self.Health.colorDisconnected = true
	self.Health.frequentUpdates = true

	self.Power = CreateFrame("StatusBar", nil, self)
	self.Power:SetHeight(9.5)
	self.Power:SetStatusBarTexture(texturebar)
	self.Power:SetStatusBarColor(.25, .25, .35)
	
	self.Power:SetPoint("LEFT", self.Health)
	self.Power:SetPoint("RIGHT", self.Health)
	self.Power:SetPoint("TOP", self.Health, "BOTTOM", 0, -1)
	self.Power:SetBackdrop(backdrophp)
	self.Power.ColotSmooth = true
	self.Power.frequentUpdates = true

	
	if unit == "targettarget" or unit == "focus" or unit == "focustarget" then
		self.Health:SetHeight(16.5)
		self.Power:SetHeight(0)
	elseif unit == "player" or unit == "target" then
		self.Health:SetHeight(27)
		self.Power:SetHeight(10.5)
	else
		self.Health:SetHeight(23)
		self.Power:SetHeight(6)
	end
	
	self.Name = setFontString(self.Health, fontn, 13) --Name
		if unit == "focus" or unit == "focustarget"  then
			self.Name:SetPoint("LEFT", self.Health, "LEFT",2,0)
			self.Name:SetWidth(80)
		elseif unit == "pet" then
			self.Name:SetPoint("TOPLEFT", self.Health, "TOPLEFT",2,-2)
			self.Name:SetWidth(80)
			
		elseif unit == "player" then
			self.Name:Hide()
		elseif unit == "target" then
			self.Name:SetPoint("RIGHT", self.Health, "RIGHT",-3,0)
			self.Name:SetWidth(170)
			self.Name:SetHeight(20)
			self.Name:SetFont(fontn, 13)
			self.Name:SetJustifyH('RIGHT')
		elseif unit == "targettarget" then
			self.Name:SetPoint("RIGHT", self.Health, "RIGHT",-3,0)
			self.Name:SetWidth(80)
			self.Name:SetJustifyH('RIGHT')
		elseif not unit then
			self.Name:SetPoint("LEFT", self.Health, "LEFT",2,0)
			self.Name:SetWidth(80)
		end
		self.Name:SetHeight(20)
		
	if unit == "player" then
		local lvl, class, race
		
		lvl = setFontString(self.Power, fontn, 12)
		lvl:SetPoint("LEFT", self.Power, "LEFT", 2, 0.5)
		self.Lvl = lvl

		class = setFontString(self.Power, fontn, 12)
		class:SetPoint("LEFT", lvl, "RIGHT",  1, 0)
		self.Class = class
	
		race = setFontString(self.Power, fontn, 12)
		race:SetPoint("LEFT", class, "RIGHT",  1, 0)
		self.Race = race

	elseif unit == "target" then
		local lvl, class, race

		race = setFontString(self.Power, fontn, 12)
		race:SetPoint("RIGHT", self.Power, "RIGHT",  -2, 0.5)
		self.Race = race
		
		class = setFontString(self.Power, fontn, 12)
		class:SetPoint("RIGHT", race, "LEFT",  -1, 0)
		self.Class = class
		
		lvl = setFontString(self.Power, fontn, 12)
		lvl:SetPoint("RIGHT", class, "LEFT", -2, 0)
		self.Lvl = lvl
	end
	
	self.Health.value = setFontString(self.Health, fontn, 13)
	if unit == "player" or unit == "focus"  or unit == "focustarget" or not unit then
		self.Health.value:SetPoint("RIGHT", -3,0)
	elseif unit == "pet" then
		self.Health.value:SetPoint("TOPRIGHT",self.Health, "TOPRIGHT", -3,-2)
	else 
		self.Health.value:SetPoint("LEFT", 2, 0)
	end
	self.Health.value:SetHeight(20)
	if(unit=="player" or unit=="target") then
		self.Power.value = setFontString(self.Power, fontn, 12)
		if(unit=="player") then
			self.Power.value:SetPoint("RIGHT", self.Power, "RIGHT", -3, 0)
		elseif(unit=="target") then
			self.Power.value:SetPoint("LEFT", self.Power, "LEFT", 2,0)
		end
	end
	
	if(unit=="player" or unit=="target" or unit == "focustarget") then
		local icon = self.Health:CreateTexture(nil, "OVERLAY")
		icon:SetHeight(18)
		icon:SetWidth(18)
		icon:SetPoint("TOP", self, 0, 5)
		icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
		self.RaidIcon = icon
	end
	if IsAddOnLoaded("oUF_WeaponEnchant") then
			self.Enchant = CreateFrame("Frame", nil, self)
			self.Enchant:SetHeight(41)
			self.Enchant:SetWidth(41 * 2)
			self.Enchant:SetPoint("TOPRIGHT", self, "TOPLEFT", -2, -1)
			self.Enchant.size = 38
			self.Enchant.spacing = 2
			self.Enchant.initialAnchor = "TOPRIGHT"
			self.Enchant["growth-x"] = "LEFT"
	end
	if(unit=='player') then
		--BuffFrame:Hide()
		--TemporaryEnchantFrame:Hide()
		
		self.Debuffs = CreateFrame("Frame", nil, self)
		self.Debuffs:SetHeight(41*4)
		self.Debuffs:SetWidth(41*4)
		self.Debuffs.size = 30
		self.Debuffs.spacing = 1
		
		self.Debuffs.initialAnchor = "BOTTOMLEFT"
		self.Debuffs["growth-x"] = "RIGHT"
		self.Debuffs["growth-y"] = "UP"
		self.Debuffs:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
	
		--self.Buffs = CreateFrame("Frame", nil, self)
		--elf.Buffs:SetHeight(320)
		--self.Buffs:SetWidth(42 * 12)
		--self.Buffs.size = 42
		--self.Buffs.spacing = 5
		
		--self.Buffs:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -5, -35)
		--self.Buffs.initialAnchor = "TOPRIGHT"
		--self.Buffs["growth-x"] = "LEFT"
		--self.Buffs["growth-y"] = "DOWN"
		--self.Buffs.filter = true


			
	elseif(unit=='target') then 
		self.Auras = CreateFrame('StatusBar', nil, self)
		self.Auras:SetHeight(120)
		self.Auras:SetWidth(280)
		self.Auras:SetPoint('BOTTOMLEFT', self, 'TOPLEFT', 1, 2)
		self.Auras['growth-x'] = 'RIGHT'
		self.Auras['growth-y'] = 'UP' 
		self.Auras.initialAnchor = 'BOTTOMLEFT'
		self.Auras.spacing = 2.5
		self.Auras.size = 28
		self.Auras.gap = true
		self.Auras.numBuffs = 18 
		self.Auras.numDebuffs = 18 
		self.sortAuras = {}
		self.sortAuras.selfFirst = true
	elseif(unit=='pet') then 
		self.Auras = CreateFrame('StatusBar', nil, self)
		self.Auras:SetHeight(100)
		self.Auras:SetWidth(130)
		self.Auras:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, -2)
		self.Auras['growth-x'] = 'RIGHT'
		self.Auras['growth-y'] = 'DOWN'
		self.Auras.initialAnchor = 'TOPLEFT' 
		self.Auras.spacing = 1.0
		self.Auras.size = 24
		self.Auras.gap = true
		self.Auras.numBuffs = 15
		self.Auras.numDebuffs = 40
	end
	
	if (self:GetParent():GetName():match'oUF_Party' or unit == 'player') then 
		self.Leader = self.Health:CreateTexture(nil, 'OVERLAY')
		self.Leader:SetHeight(17)
		self.Leader:SetWidth(17)
		self.Leader:SetPoint('TOPLEFT', -2, 12)
		self.Leader:SetTexture('Interface\\GroupFrame\\UI-Group-LeaderIcon')
	end
	
	if(unit == 'player') then
		self.Combat = self.Health:CreateTexture(nil, 'OVERLAY')
		self.Combat:SetHeight(17)
		self.Combat:SetWidth(17)
		self.Combat:SetPoint('TOPRIGHT', 2, 12)
		self.Combat:SetTexture('Interface\\CharacterFrame\\UI-StateIcon')
		self.Combat:SetTexCoord(0.58, 0.90, 0.08, 0.41)
	end

--CastBars
	if(unit == 'player' and tCastbar) then
		local colorcb
		local _,classcb = UnitClass(unit)
		colorcb = oUF.colors.class[classcb]

		self.Castbar = CreateFrame('StatusBar', nil, self)
		self.Castbar:SetPoint('TOP', UIParentr, 'CENTER', 0, -126)
		self.Castbar:SetStatusBarTexture(texturebar)
		self.Castbar:SetStatusBarColor(colorcb[1], colorcb[2], colorcb[3])
		self.Castbar:SetBackdrop(backdrophp)
		self.Castbar:SetBackdropColor(colorcb[1]/3, colorcb[2]/3, colorcb[3]/3)
		self.Castbar:SetHeight(19)
		self.Castbar:SetWidth(300)
		
		self.Castbar.Spark = self.Castbar:CreateTexture(nil,'OVERLAY')
		self.Castbar.Spark:SetBlendMode("ADD")
		self.Castbar.Spark:SetHeight(55)
		self.Castbar.Spark:SetWidth(27)
		self.Castbar.Spark:SetVertexColor(colorcb[1], colorcb[2], colorcb[3])
		
		self.Castbar.Text = setFontString(self.Castbar, fontn, 13)
		self.Castbar.Text:SetPoint('LEFT', self.Castbar, 2, 0)

		self.Castbar.Time = setFontString(self.Castbar, fontn, 13)
		self.Castbar.Time:SetPoint('RIGHT', self.Castbar, -2, 0)
		self.Castbar.CustomTimeText = OverrideCastbarTime
		self.Castbar.CustomDelayText = OverrideCastbarDelay
		
		self.Castbar2 = CreateFrame('StatusBar', nil, self.Castbar)
		self.Castbar2:SetPoint('BOTTOMRIGHT', self.Castbar, 'BOTTOMRIGHT', 4, -4)
		self.Castbar2:SetPoint('TOPLEFT', self.Castbar, 'TOPLEFT', -4, 4)
		self.Castbar2:SetBackdrop(backdrop)
		self.Castbar2:SetBackdropColor(0,0,0,1)
		self.Castbar2:SetHeight(27)
		self.Castbar2:SetFrameLevel(0)
		
		self.Castbar.SafeZone = self.Castbar:CreateTexture(nil,'BACKGROUND')
		self.Castbar.SafeZone:SetPoint('TOPRIGHT')
		self.Castbar.SafeZone:SetPoint('BOTTOMRIGHT')
		self.Castbar.SafeZone:SetHeight(20)
		self.Castbar.SafeZone:SetTexture(texturebar)
		self.Castbar.SafeZone:SetVertexColor(1,1,.01,0.5)
		--]]
	end
	if(unit == 'target' and tCastbar) then
		self.Castbar = CreateFrame('StatusBar', nil, self)
		self.Castbar:SetPoint('TOP', UIParentr, 'CENTER', 0, -106)
		self.Castbar:SetStatusBarTexture(texturebar)
		self.Castbar:SetStatusBarColor(.81,.81,.25)
		self.Castbar:SetBackdrop(backdrophp)
		self.Castbar:SetBackdropColor(.81/3,.81/3,.25/3)
		self.Castbar:SetHeight(11)
		self.Castbar:SetWidth(300)
		
		self.Castbar.Text = setFontString(self.Castbar, fontn, 12)
		self.Castbar.Text:SetPoint('LEFT', self.Castbar, 2, 0)

		self.Castbar.Time = setFontString(self.Castbar, fontn, 12)
		self.Castbar.Time:SetPoint('RIGHT', self.Castbar, -2, 0)
		self.Castbar.CustomTimeText = OverrideCastbarTime
		
		self.Castbar2 = CreateFrame('StatusBar', nil, self.Castbar)
		self.Castbar2:SetPoint('BOTTOMRIGHT', self.Castbar, 'BOTTOMRIGHT', 4, -4)
		self.Castbar2:SetPoint('TOPLEFT', self.Castbar, 'TOPLEFT', -4, 4)
		self.Castbar2:SetBackdrop(backdrop)
		self.Castbar2:SetBackdropColor(0,0,0,1)
		self.Castbar2:SetHeight(21)
		self.Castbar2:SetFrameLevel(0)
		
		self.Castbar.Spark = self.Castbar:CreateTexture(nil,'OVERLAY')
		self.Castbar.Spark:SetBlendMode("Add")
		self.Castbar.Spark:SetHeight(35)
		self.Castbar.Spark:SetWidth(25)
		self.Castbar.Spark:SetVertexColor(.69,.31,.31)
	end
	if(unit == 'focus'  and tCastbar) then
		self.Castbar = CreateFrame('StatusBar', nil, self)
		self.Castbar:SetPoint('TOP', UIParentr, 'CENTER', 0, -155)
		self.Castbar:SetStatusBarTexture(texturebar)
		self.Castbar:SetStatusBarColor(.79,.41,.31)
		self.Castbar:SetBackdrop(backdrophp)
		self.Castbar:SetBackdropColor(.79/3,.41/3,.31/3)
		self.Castbar:SetHeight(11)
		self.Castbar:SetWidth(280)
		
		self.Castbar.Text = setFontString(self.Castbar, fontn, 12)
		self.Castbar.Text:SetPoint('LEFT', self.Castbar, 2, 0)

		self.Castbar.Time = setFontString(self.Castbar, fontn, 12)
		self.Castbar.Time:SetPoint('RIGHT', self.Castbar, -2, 0)
		self.Castbar.CustomTimeText = OverrideCastbarTime
		
		self.Castbar2 = CreateFrame('StatusBar', nil, self.Castbar)
		self.Castbar2:SetPoint('BOTTOMRIGHT', self.Castbar, 'BOTTOMRIGHT', 4, -4)
		self.Castbar2:SetPoint('TOPLEFT', self.Castbar, 'TOPLEFT', -4, 4)
		self.Castbar2:SetBackdrop(backdrop)
		self.Castbar2:SetBackdropColor(0,0,0,1)
		self.Castbar2:SetHeight(21)
		self.Castbar2:SetFrameLevel(0)
		
		self.Castbar.Spark = self.Castbar:CreateTexture(nil,'OVERLAY')
		self.Castbar.Spark:SetBlendMode("Add")
		self.Castbar.Spark:SetHeight(35)
		self.Castbar.Spark:SetWidth(25)
		self.Castbar.Spark:SetVertexColor(.69,.31,.31)
	end
--[[Swing
	if(IsAddOnLoaded('oUF_Swing')) and (unit == 'player') then
			local colorcb
			local _,classcb = UnitClass(unit)
			colorcb = oUF.colors.class[classcb]
			
			self.Swing = CreateFrame('StatusBar', nil, self)
			self.Swing:SetPoint('TOPLEFT', self.Castbar, 'TOPLEFT', 0, 13)
			self.Swing:SetPoint('BOTTOMRIGHT', self.Castbar, 'TOPRIGHT', 0, 5)
			self.Swing:SetStatusBarTexture(texturebar)
			self.Swing:SetStatusBarColor(colorcb[1], colorcb[2], colorcb[3])
			self.Swing:SetBackdrop(backdrophp)
			self.Swing:SetBackdropColor(colorcb[1]/2.5, colorcb[2]/2.5, colorcb[3]/2.5)

			self.Swing.Text = setFontString(self.Swing, fontn, 12)
			self.Swing.Text:SetPoint('CENTER', 0, 1)
			self.Swing.Text:SetTextColor(.84,.75,.65) 
			
			self.Swing.bg = self.Swing:CreateTexture(nil, 'BORDER')
			self.Swing.bg:SetAllPoints(self.Swing)
			self.Swing.bg:SetTexture(texturebar)
			self.Swing.bg:SetVertexColor(.15,.15,.15)
		end
--]]
--ComboPoints	
	if(unit == 'target') then
		self.CPoints = {}
		--self.CPoints.unit = 'player'
		self.CPoints[1] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.CPoints[1]:SetHeight(17)
		self.CPoints[1]:SetWidth(17)
		self.CPoints[1]:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, 0)
		self.CPoints[1]:SetTexture(bubbleTex)
		self.CPoints[1]:SetVertexColor(.33,.63,.33)

		self.CPoints[2] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.CPoints[2]:SetHeight(17)
		self.CPoints[2]:SetWidth(17)
		self.CPoints[2]:SetPoint('LEFT', self.CPoints[1], 'RIGHT', 1)
		self.CPoints[2]:SetTexture(bubbleTex)
		self.CPoints[2]:SetVertexColor(.33,.63,.33)

		self.CPoints[3] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.CPoints[3]:SetHeight(17)
		self.CPoints[3]:SetWidth(17)
		self.CPoints[3]:SetPoint('LEFT', self.CPoints[2], 'RIGHT', 1)
		self.CPoints[3]:SetTexture(bubbleTex)
		self.CPoints[3]:SetVertexColor(.67,.67,.33)

		self.CPoints[4] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.CPoints[4]:SetHeight(17)
		self.CPoints[4]:SetWidth(17)
		self.CPoints[4]:SetPoint('LEFT', self.CPoints[3], 'RIGHT', 1)
		self.CPoints[4]:SetTexture(bubbleTex)
		self.CPoints[4]:SetVertexColor(.67,.67,.33)

		self.CPoints[5] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.CPoints[5]:SetHeight(17)
		self.CPoints[5]:SetWidth(17)
		self.CPoints[5]:SetPoint('LEFT', self.CPoints[4], 'RIGHT', 1)
		self.CPoints[5]:SetTexture(bubbleTex)
		self.CPoints[5]:SetVertexColor(.69,.31,.31)	
	end
--Holy Power
	if(class == 'PALADIN' and unit == 'player') then
		self.HolyPower = {}

		self.HolyPower[1] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.HolyPower[1]:SetHeight(17)
		self.HolyPower[1]:SetWidth(17)
		self.HolyPower[1]:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, 0)
		self.HolyPower[1]:SetTexture(bubbleTex)
		self.HolyPower[1]:SetVertexColor(.67,.67,.33)

		self.HolyPower[2] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.HolyPower[2]:SetHeight(17)
		self.HolyPower[2]:SetWidth(17)
		self.HolyPower[2]:SetPoint('LEFT', self.HolyPower[1], 'RIGHT', 1)
		self.HolyPower[2]:SetTexture(bubbleTex)
		self.HolyPower[2]:SetVertexColor(.67,.67,.33)

		self.HolyPower[3] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.HolyPower[3]:SetHeight(17)
		self.HolyPower[3]:SetWidth(17)
		self.HolyPower[3]:SetPoint('LEFT', self.HolyPower[2], 'RIGHT', 1)
		self.HolyPower[3]:SetTexture(bubbleTex)
		self.HolyPower[3]:SetVertexColor(.67,.67,.33)
		
	end

 --Soul shards
	if(class == 'WARLOCK' and unit == 'player') then
		self.SoulShards = {}

		self.SoulShards[1] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.SoulShards[1]:SetHeight(17)
		self.SoulShards[1]:SetWidth(17)
		self.SoulShards[1]:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, 0)
		self.SoulShards[1]:SetTexture(bubbleTex)
		self.SoulShards[1]:SetVertexColor(.187,.45,.45)

		self.SoulShards[2] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.SoulShards[2]:SetHeight(17)
		self.SoulShards[2]:SetWidth(17)
		self.SoulShards[2]:SetPoint('LEFT', self.SoulShards[1], 'RIGHT', 1)
		self.SoulShards[2]:SetTexture(bubbleTex)
		self.SoulShards[2]:SetVertexColor(.187,.45,.45)

		self.SoulShards[3] = self.Power:CreateTexture(nil, 'OVERLAY')
		self.SoulShards[3]:SetHeight(17)
		self.SoulShards[3]:SetWidth(17)
		self.SoulShards[3]:SetPoint('LEFT', self.SoulShards[2], 'RIGHT', 1)
		self.SoulShards[3]:SetTexture(bubbleTex)
		self.SoulShards[3]:SetVertexColor(.187,.45,.45)
		
	end

--Runes
	if(class == 'DEATHKNIGHT' and unit == 'player' and tRunebar) then
		self.RuneBar = {}
		for i = 1, 6 do
			self.RuneBar[i] = CreateFrame('StatusBar', nil, self)
			if(i == 1) then
				self.RuneBar[i]:SetPoint('BOTTOMRIGHT', self, 'BOTTOMLEFT', -4, 4)
			else
				self.RuneBar[i]:SetPoint('TOPRIGHT', self.RuneBar[i-1], 'TOPLEFT', -7, 0)
			end
			self.RuneBar[i]:SetStatusBarTexture(texturebar)--(trunebar)
			--self.RuneBar[i]:SetStatusBarColor(unpack(runeloadcolors[i]))
			self.RuneBar[i]:SetHeight(39)
			self.RuneBar[i]:SetWidth(6)--(275/6 - 1.25)
			self.RuneBar[i]:SetBackdrop(backdrophp)
			self.RuneBar[i]:SetBackdropColor(.75,.75,.75)
			self.RuneBar[i]:SetMinMaxValues(0, 1)
			self.RuneBar[i]:SetOrientation("Vertical")
			self.RuneBar[i]:SetID(i)
			local runetype = GetRuneType(i)
			if(runetype) then
				self.RuneBar[i]:SetStatusBarColor(unpack(colors.runes[runetype]))
				
			end

			self.RuneBar[i].bg = CreateFrame('StatusBar', nil, self.RuneBar[i])
			self.RuneBar[i].bg:SetPoint('BOTTOMRIGHT', self.RuneBar[i], 'BOTTOMRIGHT', 4, -4)
			self.RuneBar[i].bg:SetPoint('TOPLEFT', self.RuneBar[i], 'TOPLEFT', -4, 4)
			self.RuneBar[i].bg:SetBackdrop(backdrop)
			self.RuneBar[i].bg:SetBackdropColor(0,0,0,1)
			self.RuneBar[i].bg:SetHeight(27)
			self.RuneBar[i].bg:SetFrameLevel(0)
			
		end

		RuneFrame:Hide()

		self:RegisterEvent('RUNE_TYPE_UPDATE', UpdateRuneType)
		self:RegisterEvent('RUNE_REGEN_UPDATE', UpdateRuneType)
		self:RegisterEvent('RUNE_POWER_UPDATE', UpdateRunePower)
	end

	self.PostCreateAuraIcon = PostCreateAuraIcon
	self.PostUpdateAuraIcon = PostUpdateAuraIcon
	self.UNIT_NAME_UPDATE = OverrideUpdateName
	self.PostUpdateHealth = PostUpdateHealth
	self.PostUpdatePower = PostUpdatePower
	self.PostCreateEnchantIcon = PostCreateAuraIcon
	self.PostUpdateEnchantIcons = CreateEnchantTimer
	return self
end


oUF:RegisterStyle('alekk', Style)
oUF:SetActiveStyle('alekk')



local player = oUF:Spawn("player")
player:SetPoint("CENTER", -305, -125)
local target = oUF:Spawn("target")
target:SetPoint("CENTER", 305, -125)
local pet = oUF:Spawn("pet")
pet:SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, -75)
local tt = oUF:Spawn("targettarget")
tt:SetPoint("TOPRIGHT", target, "BOTTOMRIGHT", 0, -1)
local focus = oUF:Spawn("focus")
focus:SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, -1)
local focustarget = oUF:Spawn("focustarget")
focustarget:SetPoint("TOPLEFT", focus, "TOPRIGHT", 5, 0)


local maintank = oUF:Spawn('header', 'oUF_MainTank')
maintank:SetManyAttributes('showRaid', true, 'groupFilter', 'MAINTANK', 'yOffset', -3)
maintank:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 8, 225)
maintank:SetAttribute("template", "oUF_MTT")
maintank:Show()


if tParty then
	local party = oUF:Spawn('header', 'oUF_Party')
	party:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 8, 225)
	party:SetManyAttributes('yOffset', -3, 'showParty', true)

	local partyToggle = CreateFrame('Frame')

	partyToggle:RegisterEvent('PLAYER_LOGIN')
	partyToggle:RegisterEvent('RAID_ROSTER_UPDATE')
	partyToggle:RegisterEvent('PARTY_LEADER_CHANGED')
	partyToggle:RegisterEvent('PARTY_MEMBERS_CHANGED')

	partyToggle:SetScript('OnEvent', function(self)
		if(InCombatLockdown()) then
			self:RegisterEvent('PLAYER_REGEN_ENABLED')
		else
			self:UnregisterEvent('PLAYER_REGEN_ENABLED')
			if(GetNumRaidMembers() > 1) then
				party:Hide()
			else
				party:Show()
			end
		end
	end)

end