--[[
	
	sTotemTimers
	Copyright (c) 2009, Nils Ruesch
	All rights reserved.
	
]]

if ( select(2, UnitClass("player")) == "SHAMAN" ) then
	TotemFrame:UnregisterAllEvents();
	TotemFrame:Hide();
else
	DisableAddOn("sTotemTimers");
	return;
end

--[[ KONFIGURATION - START ]]--

local FirstTotem = { "BOTTOM", UIParent, "BOTTOM", (-26*2+5)-373, 127 }; -- Das erste Totem, wo soll es stehen? /!\ Alle 5 Argumente mssen vorhanden sein /!\ 
local NextTotem = { "LEFT", "RIGHT", 3, 0 }; -- Die folgenden Totems, wo soll es am ersten/nchsten Totem stehen? /!\ Alle 4 Argumente mssen vorhanden sein /!\ 

local TotemSize = 38; -- Breite und hhe des Totems.
local TotemScale = 1; -- Breite und hhe des Totems.

local Font = "Interface\\AddOns\\sThreatMeter2\\calibrib.ttf"; -- Die Schriftart
local FontSize = 14; -- Die Schriftgre
local FontSetPoint = { "BOTTOM", "TOP", 0, 2 }; -- Die Duration, wo soll sie stehen ?
--local FontSetPoint = { "CENTER", "CENTER", 0, 0 }; -- Die Duration, wo soll sie stehen ?
--[[ KONFIGURATION - ENDE ]]--

local function TotemOnUpdate(self, elapsed)
	self.timeLeft = GetTotemTimeLeft(self.slot);
	AuraButton_UpdateDuration(self, self.timeLeft);
	if ( GameTooltip:IsOwned(self) ) then
		GameTooltip:SetTotem(self.slot);
	end
	if ( self.timeLeft < BUFF_DURATION_WARNING_TIME and not UIFrameIsFlashing(self) ) then
		UIFrameFlash(self, BUFF_FLASH_TIME_ON, BUFF_FLASH_TIME_OFF, BUFF_DURATION_WARNING_TIME);
	elseif ( self.timeLeft > BUFF_DURATION_WARNING_TIME and UIFrameIsFlashing(self) ) then
		UIFrameFlashRemoveFrame(self);
	end
end

local function TotemOnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetTotem(self.slot);
end

local function TotemOnLeave(self)
	GameTooltip:Hide();
end

local function TotemOnClick(self, ...)
	local buton = ...;
	if ( buton == "RightButton" ) then
		DestroyTotem(self.slot);
	end
end

local function OnEvent(self, event, ...)
	local haveTotem, name, startTime, duration, icon
	local button, buttonIcon;
	local slot;
	for i=1, MAX_TOTEMS, 1 do
		slot = TOTEM_PRIORITIES[i];
		haveTotem, name, startTime, duration, icon = GetTotemInfo(i);
		button = _G["sTotemTimersTotem"..slot];
		buttonIcon = _G["sTotemTimersTotem"..slot.."Icon"];
		if ( haveTotem and duration > 0 ) then
			button.slot = i;
			button.timeLeft = 0;
			buttonIcon:SetTexture(icon);
			button:SetAlpha(1);
			button:SetScript("OnUpdate", TotemOnUpdate);
			button:Show();
		else
			if ( button:IsShown() ) then
				if ( UIFrameIsFlashing(button) ) then
					UIFrameFlashRemoveFrame(button);
				end
				button:SetScript("OnUpdate", nil);
				button:Hide();
			end
		end
	end
end

local frame = CreateFrame("Frame", "sTotemTimers", UIParent);
frame.totem = {};
for i=1, MAX_TOTEMS, 1 do
	frame.totem[i] = CreateFrame("Button", "$parentTotem"..i, frame);
	frame.totem[i]:SetWidth(TotemSize);
	frame.totem[i]:SetHeight(TotemSize);
	frame.totem[i]:SetScale(TotemScale);
	frame.totem[i]:EnableMouse(1);
	if ( i == 1 ) then
		frame.totem[i]:SetPoint(FirstTotem[1], FirstTotem[2], FirstTotem[3], FirstTotem[4], FirstTotem[5]);
	else
		frame.totem[i]:SetPoint(NextTotem[1], frame.totem[i-1], NextTotem[2], NextTotem[3], NextTotem[4]);
	end
	frame.totem[i]:SetScript("OnEnter", TotemOnEnter);
	frame.totem[i]:SetScript("OnLeave", TotemOnLeave);
	frame.totem[i]:RegisterForClicks("RightButtonUp");
	frame.totem[i]:SetScript("OnClick", TotemOnClick);
	frame.totem[i]:Hide();
	
	frame.totem[i].icon = frame.totem[i]:CreateTexture("$parentIcon", "BACKGROUND");
	frame.totem[i].icon:SetAllPoints();
	frame.totem[i].icon:SetTexture("Interface\\Icons\\temp");
	frame.totem[i].icon:SetTexCoord(0, 1, 0, 1);
	

	frame.totem[i].duration = frame.totem[i]:CreateFontString("$parentDuration", "OVERLAY");
	frame.totem[i].duration:SetFont(Font, FontSize);
	frame.totem[i].duration:SetShadowOffset(1, -1);
	frame.totem[i].duration:SetVertexColor(1,1,0,1);
	frame.totem[i].duration:SetPoint(FontSetPoint[1], frame.totem[i], FontSetPoint[2], FontSetPoint[3], FontSetPoint[4]);
	
	
	
	frame.totem[i].border = frame.totem[i]:CreateTexture("$parentBorder", "OVERLAY");
	frame.totem[i].border:SetPoint("TOPLEFT", -1, 1);
	frame.totem[i].border:SetPoint("BOTTOMRIGHT", 1, -1);
	frame.totem[i].border:SetTexture("Interface\\AddOns\\sTotemTimers\\border");
	frame.totem[i].border:SetVertexColor(0.35, 0.35, 0.35, 1);
end
frame:RegisterEvent("PLAYER_TOTEM_UPDATE");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:SetScript("OnEvent", OnEvent);
